from math import log
from numpy import std,mean,median,isnan
import re

 

def _integrate(xvals,yvals):
	#xvals: lift of positions of markers
	#yvals: list of values for markers
	
	if len(xvals)!=len(yvals):
		return
	area=0.0
	for i in range(len(xvals)-1):
		t=abs(xvals[i+1]-xvals[i])/2.0*(yvals[i]+yvals[i+1])
		area+=t
	
	return area

def _integrate_around_allele(marker,positions,ind_indices,index):



	ehhs=[1]
	pos=[positions[index]]
	area1=0.0
	area2=0.0

	hapl={}
	for m in ind_indices:
		hapl[m]=[marker[m][index]]
	for k in xrange(index-1,-1,-1):
		
		for m in ind_indices:
			hapl[m].append(marker[m][k])

		ehh1=_ehh(hapl.values())

		ehhs.append(ehh1)
		pos.append(positions[k])

		if ehh1<0.05 or k==0:
			area1=_integrate(pos,ehhs)
			break
	



	hapl={}
	for m in ind_indices:
		hapl[m]=[marker[m][index]]
		
	ehhs=[1]
	pos=[positions[index]]

	for k in xrange(index+1,len(marker[0])):
			
		for m in ind_indices:				
			hapl[m].append(marker[m][k])

		ehh1=_ehh(hapl.values())

		ehhs.append(ehh1)
		pos.append(positions[k])

		if ehh1<0.05 or k==len(marker[0]):
			area2=_integrate(pos,ehhs)
			break
	


	area=area1+area2
	return area

def _wintegrate_around_allele(marker,positions,ind_indices,index,pw_identities):


	ehhs=[1]
	pos=[positions[index]]
	area1=0.0
	area2=0.0


	hapl={}
	for m in ind_indices:
		hapl[m]=[marker[m][index]]
	
	num_inds=len(hapl)

	uniqueness={}

	sum_all_id=0.0	

	for i in xrange(len(ind_indices)):
		sum_id=0.0
		for j in xrange(len(ind_indices)):
			if i==j:
				continue
			sum_id+=pw_identities[(ind_indices[i],ind_indices[j])]
			
			
		u=(1-(sum_id/float(num_inds-1)))**2
		sum_all_id+=u
		uniqueness[i]=u
	if sum_all_id==0:
		return None

	for i in xrange(num_inds):
		uniqueness[i]=uniqueness[i]/sum_all_id


	for k in xrange(index-1,-1,-1):
		
		for m in ind_indices:
			hapl[m].append(marker[m][k])

		vec=[]
		for m in ind_indices:
			vec.append(hapl[m])

		ehh1=_wehh(vec,uniqueness)

		ehhs.append(ehh1)
		pos.append(positions[k])

		if ehh1<0.05 or k==0:
			area1=_integrate(pos,ehhs)
			break
	

	hapl={}
	for m in ind_indices:
		hapl[m]=[marker[m][index]]

	ehhs=[1]
	pos=[positions[index]]

	for k in xrange(index+1,len(marker[0])):
			
		for m in ind_indices:
			hapl[m].append(marker[m][k])

		vec=[]
		for m in ind_indices:
			vec.append(hapl[m])

		ehh1=_wehh(vec,uniqueness)

		ehhs.append(ehh1)
		pos.append(positions[k])

		if ehh1<0.05 or k==0:
			area2=_integrate(pos,ehhs)
			break
	



	area=area1+area2
	return area
	
def _ehh(hapl1):
	#hapl1: list of lists of marker-values; one list per accession
	
	num_inds=len(hapl1)

	if len(hapl1)<=1:
		return 0.0
	
	hapl2=[]
		
	for h in hapl1:
		#converting lists to tuples(hashable)
		t=tuple(h)
		hapl2.append(t)
	
	d={}

	
	
	for h in hapl2:
		if d.has_key(h):
			d[h]+=1.0#/num_inds
		else:
			d[h]=1.0#/num_inds
	
	prob=0.0
	l=len(hapl2)
	
	for v in d.values():
		prob+=v/l*(v-1)/(l-1)
		#prob+=v**2
	
	return prob

def _wehh(hapl1,uniqueness):
	#hapl1: list of lists of marker-values; one list per accession

	num_inds=len(hapl1)	

	if num_inds<=1:
		return 0.0



	hapl2=[]
	for h in hapl1:
		#converting lists to tuples(hashable)
		t=tuple(h)
		hapl2.append(t)
	
	d={}
	
	
	for i in xrange(num_inds):
		if d.has_key(hapl2[i]):
			d[hapl2[i]]+=uniqueness[i]*num_inds
		else:
			d[hapl2[i]]=uniqueness[i]*num_inds
	
	prob=0.0
	l=len(hapl2)
	
	for v in d.values():
		prob+=v/l*(v-1)/(l-1)
		#prob+=v**2
	
	return prob
	
def _compute_unstd_ihs(marker,positions):
	#marker: list of lists (one list per individual)
	#postions: marker positions on a genetic map

	num_inds=len(marker)
	num_snps=len(positions)

	from math import log

	unstd_ihs=[]
	mafs=[]

	for i in xrange(num_snps):
		#i snp index

		allele_dic={}
		for j in xrange(num_inds):
			#j ind index

			if not allele_dic.has_key(marker[j][i]):
				allele_dic[marker[j][i]]=[j]
			else:
				allele_dic[marker[j][i]].append(j)
		
		
		
		if len(allele_dic.keys())!=2:
			unstd_ihs.append(None)
			mafs.append(None)
			continue

		keys=allele_dic.keys()
		keys.sort()

		a1=allele_dic.keys()[0]
		a2=allele_dic.keys()[1]

		num_allele1=len(allele_dic[a1])
		num_allele2=len(allele_dic[a2])
		
		if allele_dic.has_key(1) or allele_dic.has_key('1'):

			maf=num_allele2/float(num_inds)
			mafs.append(maf)	
			
		else:
			#num_allele1=len(allele_dic[a1])
			#num_allele2=len(allele_dic[a2])
			#maf=min(num_allele1/float(num_inds),num_allele2/float(num_inds))
			#mafs.append(maf)
			print >> sys.stderr, 'Wrong allele coding!!! Use 0 for the ancestral allele and 1 for the derived.'	
			return

		if maf < 0.05 or num_allele1 < 2 or num_allele2 < 2:
			unstd_ihs.append(None)
			continue

		hapl={}
		for k in xrange(num_allele1):
			
			hapl[allele_dic[a1][k]]=marker[allele_dic[a1][k]][i]

		area1=_integrate_around_allele(marker,positions,allele_dic[a1],i)
		if area1==0.0:
			unstd_ihs.append(0)
			continue

		area2=_integrate_around_allele(marker,positions,allele_dic[a2],i)
		if area2==0.0:
			unstd_ihs.append(0)
			continue

		#if num_allele1/float(num_inds)<num_allele2/float(num_inds):
		#	ihs=log(area1/area2)
		#else:
		#	ihs=log(area2/area1)
		ihs=log(area1/area2)
		
		unstd_ihs.append(ihs)
	
	return tuple(unstd_ihs),tuple(mafs)

def _compute_unstd_wihs(marker,positions,pw_identities):
	#marker: list of lists (one list per individual)
	#postions: marker positions on a genetic map

	from math import log

	num_inds=len(marker)
	num_snps=len(positions)

	unstd_wihs=[]
	mafs=[]

	for i in xrange(num_snps):
		#i snp index

		allele_dic={}
		for j in xrange(num_inds):
			#j ind index
			
			if not allele_dic.has_key(marker[j][i]):
				allele_dic[marker[j][i]]=[j]
			else:
				allele_dic[marker[j][i]].append(j)
		
		

		if len(allele_dic.keys())!=2:
			unstd_wihs.append(None)
			mafs.append(None)
			continue

		keys=allele_dic.keys()
		keys.sort()

		a1=keys[0]
		a2=keys[1]

		num_allele1=len(allele_dic[a1])
		num_allele2=len(allele_dic[a2])
		
		if allele_dic.has_key(1) or allele_dic.has_key('1'):

			maf=num_allele2/float(num_inds)
			mafs.append(maf)	
			
		else:
			#num_allele1=len(allele_dic[a1])
			#num_allele2=len(allele_dic[a2])
			#maf=min(num_allele1/float(num_inds),num_allele2/float(num_inds))
			#mafs.append(maf)
			print >> sys.stderr, 'Wrong allele coding!!! Use 0 for the ancestral allele and 1 for the derived.'	
			return	

		if maf < 0.05 or num_allele1 < 2 or num_allele2 < 2:
			unstd_wihs.append(None)
			continue

		hapl={}
		for k in xrange(num_allele1):
			hapl[allele_dic[a1][k]]=marker[allele_dic[a1][k]][i]

		area1=_wintegrate_around_allele(marker,positions,allele_dic[a1],i,pw_identities)
		if area1==0.0 or area1==None:
			unstd_wihs.append(0)
			continue

		area2=_wintegrate_around_allele(marker,positions,allele_dic[a2],i,pw_identities)
		if area2==0.0 or area2==None:
			unstd_wihs.append(0)
			continue

		#if num_allele1/float(num_inds)<num_allele2/float(num_inds):
		#	wihs=log(area1/area2)
		#else:
		#	wihs=log(area2/area1)
		wihs=log(area1/area2)
		
		unstd_wihs.append(wihs)
	
	return tuple(unstd_wihs),tuple(mafs)


def _compute_unstd_wihs_window(marker,positions,window_size):
	#marker: list of lists (one list per individual)
	#postions: marker positions on a genetic map

	from math import log

	num_inds=len(marker)
	num_snps=len(positions)

	unstd_wihs=[]
	mafs=[]

	for i in xrange(num_snps):
		#i snp index

		p=positions[i]

		window_start_index=i
		window_end_index=i

		for j in xrange(0,i):
			if positions[j]>=p-window_size:
				window_start_index=j
				break

		for j in xrange(i+1,num_snps):
			if positions[j]>=p+window_size:
				window_end_index=j
				break
		#print i,window_start_index,window_end_index,p

		window_marker=[]

		for j in xrange(num_inds):
			window_marker.append(marker[j][window_start_index:window_end_index])


		pw_identities={}
		for i in xrange(num_inds):
			pw_identities[(i,i)]=1.0
			for j in xrange(i+1,num_inds):

				id_count=0
				
				for k in xrange(len(window_marker[0])):
					if marker[i][k]==marker[j][k]:
						id_count+=1
				identity=float(id_count)/len(window_marker[0])
				pw_identities[(i,j)]=identity
				pw_identities[(j,i)]=identity

		allele_dic={}
		for j in xrange(num_inds):
			#j ind index
			
			if not allele_dic.has_key(marker[j][i]):
				allele_dic[marker[j][i]]=[j]
			else:
				allele_dic[marker[j][i]].append(j)
		
		

		if len(allele_dic.keys())!=2:
			unstd_wihs.append(None)
			mafs.append(None)
			continue

		a1=allele_dic.keys()[0]
		a2=allele_dic.keys()[1]

		num_allele1=len(allele_dic[a1])
		num_allele2=len(allele_dic[a2])
		
		if allele_dic.has_key(1):

			num_allele1=len(allele_dic[1])
			num_allele2=len(allele_dic[0])
			a1=1
			a2=0


			maf=num_allele1/float(num_inds)
			mafs.append(maf)	
			
		else:
			num_allele1=len(allele_dic[a1])
			num_allele2=len(allele_dic[a2])
			maf=min(num_allele1/float(num_inds),num_allele2/float(num_inds))
			mafs.append(maf)	

		if maf < 0.05 or num_allele1 < 2 or num_allele2 < 2:
			unstd_wihs.append(None)
			continue

		hapl={}
		for k in xrange(num_allele1):
			hapl[allele_dic[a1][k]]=marker[allele_dic[a1][k]][i]

		area1=_wintegrate_around_allele(marker,positions,allele_dic[a1],i,pw_identities)
		if area1==0.0 or area1==None:
			unstd_wihs.append(0)
			continue

		area2=_wintegrate_around_allele(marker,positions,allele_dic[a2],i,pw_identities)
		if area2==0.0 or area2==None:
			unstd_wihs.append(0)
			continue

		if num_allele1/float(num_inds)<num_allele2/float(num_inds):
			wihs=log(area1/area2)
		else:
			wihs=log(area2/area1)

		
		unstd_wihs.append(wihs)
	
	return tuple(unstd_wihs),tuple(mafs)


def _standardize_ihs(unstd_ihs,mafs):
	#input minor allele frequencies and unstandardized ihs values

	from numpy import std,mean,median,isnan

	if len(unstd_ihs)!=len(mafs):
		return

	freq_dic={}

	maf_count=0

	for i in xrange(len(mafs)):
		if mafs[i]==None:
			continue
		else:
			maf_count+=1

	if maf_count<2000:
		maf_factor=maf_count/20.
	else:
		maf_factor=100

	for i in xrange(len(unstd_ihs)):
		if unstd_ihs[i]==None:
			continue
		if isnan(unstd_ihs[i]):
			continue

		maf=int(mafs[i]*maf_factor)

		if freq_dic.has_key(maf):
			freq_dic[maf].append(unstd_ihs[i])
		else:
			freq_dic[maf]=[unstd_ihs[i]]

	medians={}
	stddev={}

	for k in freq_dic.keys():
		
		medians[k]=mean(freq_dic[k])
		stddev[k]=std(freq_dic[k])

	std_ihs=[]




	for i in xrange(len(unstd_ihs)):
		if unstd_ihs[i]==None or isnan(unstd_ihs[i]):
			std_ihs.append(None)
			continue
		maf=int(mafs[i]*maf_factor)
		if stddev[maf]==0:
			std_ihs.append(None)
			continue
		
		ihs=(unstd_ihs[i]-medians[maf])/stddev[maf]
		std_ihs.append(ihs)
	
	return tuple(std_ihs)




def compute_ihs(marker,positions,use_pp=0):
	#marker - list of lists(chromosomes) of lists(individuals) of alleles
	#positions - list of lists of lists of positions (on a genetic map)
	unstd=[]
	mafs=[]

	if use_pp:
		import pp

		job_server=pp.Server()
		ncpus=job_server.get_ncpus()

		jobs=[]

		count=0
		for c in xrange(len(marker)):

			if count<ncpus:
				job=job_server.submit(_compute_unstd_ihs,(marker[c],positions[c]),(_ehh,_integrate_around_allele,_integrate),("numpy","math"))
				jobs.append(job)
				#print len(positions[c])
				count+=1
			else:
				for job in jobs:
					unstd_temp,mafs_temp=job()
					#print len(unstd_temp)		
					unstd+=unstd_temp
					mafs+=mafs_temp
				jobs=[]
				count=1
				job=job_server.submit(_compute_unstd_ihs,(marker[c],positions[c]),(_ehh,_wintegrate_around_allele,_integrate),("numpy","math"))
				jobs.append(job)
		for job in jobs:
			unstd_temp,mafs_temp=job()
			unstd+=unstd_temp
			mafs+=mafs_temp

	else:

		for c in xrange(len(marker)):
				 
			unstd_temp,mafs_temp=_compute_unstd_ihs(marker[c],positions[c])
			unstd+=unstd_temp
			mafs+=mafs_temp


	return _standardize_ihs(unstd,mafs)

def compute_wihs(marker,positions,pw_identities=0,use_pp=0):
        #marker - list of lists(chromosomes) of lists(individuals) of alleles
        #positions - list of lists of lists of positions (on a genetic map)
	#pw_identities - dict of pairwise identities (keys: tuples of pw indices)


	if not pw_identities:

		pw_identities={}
		for i in xrange(len(marker[0])):
			pw_identities[(i,i)]=1.0
			for j in xrange(i+1,len(marker[0])):
				num_snps_all=0
				id_count=0
				for c in xrange(len(marker)):
					num_snps=len(marker[c][j])
					num_snps_all+=num_snps
				
					for k in xrange(num_snps):
						if marker[c][i][k]==marker[c][j][k]:
							id_count+=1
				identity=float(id_count)/num_snps_all
				pw_identities[(i,j)]=identity
				pw_identities[(j,i)]=identity

	

	unstd=[]
	mafs=[]

	if use_pp:
		import pp

		job_server=pp.Server()
		ncpus=job_server.get_ncpus()

		jobs=[]

		count=0
		for c in xrange(len(marker)):

			if count<ncpus:
				#print len(positions[c]) 
				job=job_server.submit(_compute_unstd_wihs,(marker[c],positions[c],pw_identities),(_wehh,_wintegrate_around_allele,_integrate),("numpy","math"))
				jobs.append(job)
				count+=1
			else:
				for job in jobs:
					unstd_temp,mafs_temp=job()
					#print len(unstd_temp)
					unstd+=unstd_temp
					mafs+=mafs_temp
				jobs=[]
				count=1
				job=job_server.submit(_compute_unstd_wihs,(marker[c],positions[c],pw_identities),(_wehh,_wintegrate_around_allele,_integrate),("numpy","math"))
				jobs.append(job)
		for job in jobs:
			unstd_temp,mafs_temp=job()
			#print len(unstd_temp)
			unstd+=unstd_temp
			mafs+=mafs_temp

	else:

		for c in xrange(len(marker)):
				 
			unstd_temp,mafs_temp=_compute_unstd_wihs(marker[c],positions[c],pw_identities)
			unstd+=unstd_temp
			mafs+=mafs_temp


	return _standardize_ihs(unstd,mafs)


def compute_wihs_window(marker,positions,window_size,use_pp=0):
        #marker - list of lists(chromosomes) of lists(individuals) of alleles
        #positions - list of lists of lists of positions (on a genetic map)
	#window_size - window around focal SNP for which PW matrix is calculated




	

	unstd=[]
	mafs=[]

	if use_pp:
		import pp

		job_server=pp.Server()
		ncpus=job_server.get_ncpus()

		jobs=[]

		count=0
		for c in xrange(len(marker)):

			if count<ncpus:
				#print len(positions[c]) 
				job=job_server.submit(_compute_unstd_wihs_window,(marker[c],positions[c],window_size),(_wehh,_wintegrate_around_allele,_integrate),("numpy","math"))
				jobs.append(job)
				count+=1
			else:
				for job in jobs:
					unstd_temp,mafs_temp=job()
					#print len(unstd_temp)
					unstd+=unstd_temp
					mafs+=mafs_temp
				jobs=[]
				count=1
				job=job_server.submit(_compute_unstd_wihs_window,(marker[c],positions[c],window_size),(_wehh,_wintegrate_around_allele,_integrate),("numpy","math"))
				jobs.append(job)
		for job in jobs:
			unstd_temp,mafs_temp=job()
			#print len(unstd_temp)
			unstd+=unstd_temp
			mafs+=mafs_temp

	else:

		for c in xrange(len(marker)):
				 
			unstd_temp,mafs_temp=_compute_unstd_wihs_window(marker[c],positions[c],window_size)
			unstd+=unstd_temp
			mafs+=mafs_temp


	return _standardize_ihs(unstd,mafs)

def _1pop_integrate_around_allele(marker,positions,index):



	
	pos=[positions[index]]
	area1=0.0
	area2=0.0
	ind_indices=range(len(marker))

	hapl={}
	for m in ind_indices:
		hapl[m]=[marker[m][index]]

	vec=[]
	for m in ind_indices:
		vec.append(hapl[m])

	ehh0=_ehh(vec)
	ehhs=[ehh0]
	for k in xrange(index-1,-1,-1):
		
		for m in ind_indices:
			hapl[m].append(marker[m][k])

		ehh1=_ehh(hapl.values())/ehh0
		
		ehhs.append(ehh1)
		pos.append(positions[k])

		if ehh1<0.05 or k==0:
			area1=_integrate(pos,ehhs)
			
			break
	



	hapl={}
	for m in ind_indices:
		hapl[m]=[marker[m][index]]
		
	ehhs=[ehh0]
	pos=[positions[index]]
	
	for k in xrange(index+1,len(marker[0])):
			
		for m in ind_indices:				
			hapl[m].append(marker[m][k])

		ehh1=_ehh(hapl.values())/ehh0

		ehhs.append(ehh1)
		pos.append(positions[k])
		
		
		if ehh1<0.05 or k==len(marker[0])-1:
			area2=_integrate(pos,ehhs)
			
			break
	


	area=area1+area2
	return area

def _1pop_wintegrate_around_allele(marker,positions,index,uniqueness):


	
	pos=[positions[index]]
	area1=0.0
	area2=0.0

	ind_indices=range(len(marker))

	hapl={}
	for m in ind_indices:
		hapl[m]=[marker[m][index]]
	
	num_inds=len(hapl)

	vec=[]
	for m in ind_indices:
		vec.append(hapl[m])

	ehh0=_wehh(vec,uniqueness)

	ehhs=[ehh0]

	for k in xrange(index-1,-1,-1):
		
		for m in ind_indices:
			hapl[m].append(marker[m][k])

		vec=[]
		for m in ind_indices:
			vec.append(hapl[m])

		ehh1=_wehh(vec,uniqueness)/ehh0

		ehhs.append(ehh1)
		pos.append(positions[k])

		if ehh1<0.05 or k==0:
			area1=_integrate(pos,ehhs)
			break
	

	hapl={}
	for m in ind_indices:
		hapl[m]=[marker[m][index]]

	ehhs=[ehh0]
	pos=[positions[index]]

	for k in xrange(index+1,len(marker[0])):
			
		for m in ind_indices:
			hapl[m].append(marker[m][k])

		vec=[]
		for m in ind_indices:
			vec.append(hapl[m])

		ehh1=_wehh(vec,uniqueness)/ehh0

		ehhs.append(ehh1)
		pos.append(positions[k])

		if ehh1<0.05 or k==len(marker[0])-1:
			area2=_integrate(pos,ehhs)
			break
	



	area=area1+area2
	return area


def _compute_1pop_ihs(marker,positions):
	#marker: list of lists (one list per individual)
	#postions: marker positions on a genetic map

	num_inds=len(marker)
	num_snps=len(positions)

	pop_ihs=[]


	for i in xrange(num_snps):
		#i snp index


		ihs=_1pop_integrate_around_allele(marker,positions,i)
		if ihs==0.0 or ihs==None:
			pop_ihs.append(0)
			continue


		
		pop_ihs.append(ihs)
	
	return tuple(pop_ihs)

def _compute_1pop_wihs(marker,positions,uniqueness):
	#marker: list of lists (one list per individual)
	#postions: marker positions on a genetic map

	num_inds=len(marker)
	num_snps=len(positions)

	pop_wihs=[]


	for i in xrange(num_snps):
		#i snp index

		wihs=_1pop_wintegrate_around_allele(marker,positions,i,uniqueness)
		if wihs==0.0 or wihs==None:
			pop_wihs.append(0)
			continue

		
		pop_wihs.append(wihs)
	
	return tuple(pop_wihs)


def compute_cross_ihs(marker1,marker2,positions):
	unstd=[]

	for c in xrange(len(marker1)):		 
		unstd_temp1=_compute_1pop_ihs(marker1[c],positions[c])
		
		unstd_temp2=_compute_1pop_ihs(marker2[c],positions[c])
		
		for i in xrange(len(unstd_temp1)):
			if isnan(unstd_temp1[i]) or isnan(unstd_temp2[i]) or unstd_temp1[i]==None or unstd_temp2[i] ==None or unstd_temp1[i]==0.0 or unstd_temp2[i]==0.0:
				unstd.append(0.0)
			else:
				unstd.append(log(unstd_temp1[i]/unstd_temp2[i]))

	m=mean(unstd)
	s=std(unstd)
	
	cross_ihs=[]

	for i in xrange(len(unstd)):
		cross_ihs.append((unstd[i]-m)/s)

	return cross_ihs


def compute_cross_wihs(marker1,marker2,positions):

	
	pw_identities1={}
	for i in xrange(len(marker1[0])):
		pw_identities1[(i,i)]=1.0
		for j in xrange(i+1,len(marker1[0])):
			num_snps_all=0
			id_count=0
			for c in xrange(len(marker1)):
				num_snps=len(marker1[c][j])
				num_snps_all+=num_snps
				
				for k in xrange(num_snps):
					if marker1[c][i][k]==marker1[c][j][k]:
						id_count+=1
			identity=float(id_count)/num_snps_all
			pw_identities1[(i,j)]=identity
			pw_identities1[(j,i)]=identity

	uniqueness1={}

	sum_all_id=0.0

	ind_indices=range(len(marker1[0]))
	num_inds=len(ind_indices)

	for i in xrange(len(ind_indices)):
		sum_id=0.0
		for j in xrange(len(ind_indices)):
			if i==j:
				continue
			sum_id+=pw_identities1[(ind_indices[i],ind_indices[j])]
		u=(1-(sum_id/float(num_inds-1)))**2
		sum_all_id+=u
		uniqueness1[i]=u

	for i in xrange(num_inds):
		uniqueness1[i]=uniqueness1[i]/sum_all_id


	pw_identities2={}
	for i in xrange(len(marker2[0])):
		pw_identities2[(i,i)]=1.0
		for j in xrange(i+1,len(marker2[0])):
			num_snps_all=0
			id_count=0
			for c in xrange(len(marker2)):
				num_snps=len(marker2[c][j])
				num_snps_all+=num_snps
				
				for k in xrange(num_snps):
					if marker2[c][i][k]==marker2[c][j][k]:
						id_count+=1
			identity=float(id_count)/num_snps_all
			pw_identities2[(i,j)]=identity
			pw_identities2[(j,i)]=identity

	uniqueness2={}

	sum_all_id=0.0

	ind_indices=range(len(marker2[0]))

	for i in xrange(len(ind_indices)):
		sum_id=0.0
		for j in xrange(len(ind_indices)):
			if i==j:
				continue
			sum_id+=pw_identities2[(ind_indices[i],ind_indices[j])]
		u=(1-(sum_id/float(num_inds-1)))**2
		sum_all_id+=u
		uniqueness2[i]=u

	for i in xrange(num_inds):
		uniqueness2[i]=uniqueness2[i]/sum_all_id


	unstd=[]

	for c in xrange(len(marker1)):		 
		unstd_temp1=_compute_1pop_wihs(marker1[c],positions[c],uniqueness1)
		unstd_temp2=_compute_1pop_wihs(marker2[c],positions[c],uniqueness2)
		for i in xrange(len(unstd_temp1)):
			if isnan(unstd_temp1[i]) or isnan(unstd_temp2[i]) or unstd_temp1[i]==None or unstd_temp2[i] ==None or unstd_temp1[i]==0.0 or unstd_temp2[i]==0.0:
				unstd.append(0.0)
			else:
				unstd.append(log(unstd_temp1[i]/unstd_temp2[i]))

	m=mean(unstd)
	s=std(unstd)

	cross_wihs=[]

	for i in xrange(len(unstd)):
		cross_wihs.append((unstd[i]-m)/s)

	return cross_wihs

def test():

	from parse_fregene import parse_data
	import gc,datetime,re
	import numpy
	from phs import compute_phs

	"""num_inds=100
	num_snps=10000
	pos=range(1,num_snps+1)

		

	import numpy
	marker=[]

	for i in xrange(num_inds):
		marker.append(list(numpy.random.randint(0,2,num_snps)))

	#compute_ihs(marker,pos)
	#compute_wihs(marker,pos)
	
	print 'done'"""

	sample=1000

	"""fname='fregene_test_ds/PopnB_1Mb/Output_selected/rin_POPNB_1Mb_selected.xml'
	f=open(fname)
	lines=f.readlines()
	f.close()


	
	now1=datetime.datetime.now()
	snps,pos=parse_data(lines)
	now2=datetime.datetime.now()
	print (now2-now1).seconds
	gc.collect()

	f=open('pos.txt','w')
	for p in pos:
		f.write('%d\n'%p)
	f.close()

	f=open('snps.txt','w')
	for ind in snps:
		for s in ind:
			f.write('%d\t' %s)
		f.write('\n')
	f.close()"""

	f=open('/home/tguenther/projects/selection_test/pos.txt')
	f=open('/home/tguenther/tools/Fregene/Example/pos2.txt')
	lines=f.readlines()
	f.close()
	pos=map(int,lines)

	f=open('/home/tguenther/projects/selection_test/snps.txt')
	f=open('/home/tguenther/tools/Fregene/Example/snps2.txt')
	lines=f.readlines()
	f.close()
	snps=[]
	for l in lines:
		split=re.split('\t',l.rstrip())
		snps.append(map(int,split))

	num_inds=len(snps)
	print len(pos)

	indices=numpy.random.randint(0,num_inds,sample)

	new_snps=[]
	for i in indices:
		new_snps.append(snps[i])
	new_snps=tuple(new_snps)

	now1=datetime.datetime.now()
	ihs=compute_ihs([new_snps],[pos])
	now2=datetime.datetime.now()
	print (now2-now1).seconds
	gc.collect()

	f=open('/home/tguenther/projects/selection_test/ihs2.txt','w')
	for i in xrange(len(ihs)):
		if ihs[i]==None:
			continue
		f.write('%d\t' %pos[i])
		f.write('%f' %ihs[i])
		f.write('\n')
	f.close()
	

	now3=datetime.datetime.now()
	wihs=compute_wihs([new_snps],[pos])
	now4=datetime.datetime.now()
	print (now4-now3).seconds
	gc.collect()

	f=open('/home/tguenther/projects/selection_test/wihs2.txt','w')
	for i in xrange(len(wihs)):
		if wihs[i]==None:
			continue
		f.write('%d\t' %pos[i])
		f.write('%f' %wihs[i])
		f.write('\n')
	f.close()
	
	

	"""now3=datetime.datetime.now()
	phs=compute_phs([new_snps],[pos])
	now4=datetime.datetime.now()
	print (now4-now3).seconds
	gc.collect()
	
	f=open('phs.txt','w')
	for c in xrange(len(phs)):
		for i in xrange(len(phs[c])):
			for x in phs[c][i].keys():
				f.write('%d\t' %pos[i])
				f.write('%d\t' %x)
				f.write('%f' %phs[c][i][x])
				f.write('\n')
	f.close()"""

	
if __name__ == '__main__':
	test()
	
